/*****************************************************************************
     This file is part of the Changedfiles system. 

     Copright (C)2001  Michael L. Welles

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************/
/*  $Id: action_queue.cpp,v 1.22 2002/11/05 01:38:49 mike Exp $ */

#include "action_queue.h" 
#include "command_action.h"
#include "action_exception.h"

#include <string> 
#include <list> 
#include <map> 
#include <exception>
#include <stdexcept>
#include <pthread.h>
 
using std::string; 


namespace
{ 

  int REST_INTERVAL = 200; 
  pthread_mutex_t  m_queue_mutex =  PTHREAD_MUTEX_INITIALIZER; 
  extern "C" void *action_thread_startup( void * ); 
  void *action_thread_startup( void * _action_queue )
  {  
    ActionQueue *cq = (ActionQueue *)_action_queue; 
    cq->action_loop();
    return cq;
  }; 

}; 


ActionQueue::ActionQueue( Rule* rule ) 
{
  m_logger = Logger::get_instance(); 
  m_rule = rule; 
  m_rulename = rule->get_name();

  m_logger->log( Logger::INFO, "Queue created for regex " + this->m_rule->get_name() );
} 

ActionQueue::~ActionQueue( ) 
{
  m_logger = Logger::get_instance(); 
  m_logger->log( Logger::INFO, "Queue exiting for " + this->m_rulename );
  delete m_rule;
}

bool ActionQueue::start()
{ 
  pthread_t *cqueue_pthreadp; 
  cqueue_pthreadp = (pthread_t *)malloc( sizeof( pthread_t ) ); 
  pthread_create( cqueue_pthreadp, NULL, action_thread_startup, this ); 
  pthread_detach( *cqueue_pthreadp ); 
  free( cqueue_pthreadp ); 
  return true; 
}

bool ActionQueue::add_action( Action* bangaction ) 
{ 

  try{
    pthread_mutex_lock( &m_queue_mutex ); 
    this->m_action_list.push_back( bangaction ); 
    pthread_mutex_unlock( &m_queue_mutex ); 
  }catch( exception e ){
    string what = "error adding action,  threw: " + *(e.what());
    m_logger->log( Logger::ERR, what );
    return false;
  }

    pthread_mutex_lock( &m_queue_mutex ); 
    char size[30];
    sprintf( size, " now has %d actions pending.", m_action_list.size() );
    string size_msg = "Rule [" + this->get_rule()->get_name() + "]" + size;
    m_logger->log( Logger::INFO, size_msg );
    pthread_mutex_unlock( &m_queue_mutex ); 

  return true; 
}

void ActionQueue::action_loop()
{ 
  while( 1 )
    {

      do_next(); 
      usleep( REST_INTERVAL ); 
    }
}

/**
 * Where actions get done
 */
bool ActionQueue::do_next() 
{ 
  Action* act = NULL;
  int act_size = 0;
  bool action_success = false;

  pthread_mutex_lock( &m_queue_mutex ); 

  act_size = this->m_action_list.size();

  if ( this->m_action_list.empty() )
    { 
      action_success = true;
    }else{
      try
	{
	  act = this->m_action_list.front(); 
	  action_success = act->do_action();
	}
      catch( ActionException e )
	{
 	  string woah = "ActionQueue::caught error. ";
	  m_logger->log( Logger::ERR, woah );
	}
      catch( ... )
	{
	  m_logger->log( Logger::ERR, "Unhandled exception in " + act->get_description() );
	  action_success = true;
	}
      if( action_success )
	{
	  try
	    {
	      this->m_action_list.pop_front(); 
	    }
	  catch( exception e )
	    {
	      string err = "Couldnt pop action_list: ";
	      m_logger->log( Logger::ERR, err + e.what() );
	    }
	  catch( ... )
	    {
	      m_logger->log( Logger::ERR, "Unhandled exception" );
	    }

	  delete act;
	  char size[30];
	  sprintf( size, " now has %d actions pending.", m_action_list.size() );
	  string size_msg = "Rule [" + this->get_rule()->get_name() + "]" + size;
	  m_logger->log( Logger::INFO, size_msg );
	}
      else
	{
	  m_logger->log( Logger::DEBUG, "action failed, but we'll try again" );
	}
    }
  pthread_mutex_unlock( &m_queue_mutex ); 
  return action_success;
}

