/*****************************************************************************
     This file is part of the Changedfiles system. 

     Copright (C)2001  Michael L. Welles

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*******************************************************************************/
/*  $Id: command_action.cpp,v 1.6 2002/03/11 06:21:34 philip Exp $ */
#include "command_action.h"
#include "logger.h"
#include <string>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h> 
using std::string; 

namespace {

  const char* SHELL = "/bin/sh"; 

};

CommandAction::CommandAction( string action, int max_retry ) 
{ 
  m_command = action; 
  m_max_retry = max_retry; 
}


bool CommandAction::set_command( string action )
{ 
  m_command = action; 
  return true; 
} 

string CommandAction::get_command() 
{ 
  return m_command; 
} 

string CommandAction::get_description(){
  return "Command: " + m_command;
}


bool CommandAction::do_action() throw( ActionException )
{
  int status; 
  pid_t pid; 
  string msg;
  Logger* logger = Logger::get_instance();
  bool ret = false;
  if ( ( pid = fork() ) ) 
    {
      waitpid( pid, &status, WUNTRACED ); 
    }
  else 
    { 
      string action = this->get_command();
  
      if( execl( SHELL, SHELL, "-c", action.c_str(), NULL ) ){
	ret = true;
      }else{
	//should never get here
	exit(1); 
      }

    }

  if ( WIFEXITED( status ) ) 
    { 
      if ( WEXITSTATUS( status ) )
	{ 
	  msg = "error - Action failed, non-zero exit status. ";  
	  msg += this->get_description();
	  logger->log( Logger::ERR, msg ); 
	}
      else 
	{ 
	  msg = "info  - Action succeeded:  " + this->get_description(); 
	  logger->log( Logger::INFO, msg.c_str() ); 
	  ret = true;
	}
    }
  else if ( WIFSIGNALED( status ) )
    { 
      msg = "error - Action failed, unhandled signal passed to child.  "; 
      msg += this->get_description();
      logger->log( Logger::ERR, msg.c_str() ); 
      ret = false;
    }
  else if ( WIFSTOPPED( status ) )
    { 
      msg = "error - Action stopped: "; 
      msg += this->get_description();
      logger->log( Logger::ERR, msg.c_str() ); 
      ret = false;
    }
  else 
    { 
      msg = "error - Action failed, abnormal exit: "; 
      msg += this->get_description();
      logger->log( Logger::ERR, msg.c_str() ); 
      ret = false;
    }

  // we return success if we've exceeded our retry limit.
  // so we get removed from the queue and deleted.
  if( ! ret )
    {
      if ( ! this->add_retry() )
	{ 
	  ret = true;
	  msg = "Retry count exceeded for " + this->get_description();
	  logger->log( Logger::ERR, msg.c_str() );
	}
    }

  return ret;
}
